/* Simple huffman decoding. */

/* Copyright (C) 2009 Niels Mller
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* Huffman decoding tree is represented as a binary tree, represented
 * in a linear array of nodes. Child pointers are non-negative
 * integers used for indexing the array. For leaf nodes, left is -1
 * and right is the node value.
 */

struct huffman {
  int left;
  int right;
};

/* A tree with N leafs needs N-1 non-leaf nodes (the interior nodes
 * have a total of 2(N-1) child pointers, out of which N-2 points at
 * interior nodes, leaving 2(N-1) - (N-2) = N pointers to leafs). THe
 * total number of nodes is then 2N - 1. */

/* FIXME: Does not take into account that there may be some additional
   leaves corresponding to invali codes! For example, the (stupid)
   code A = 0, B = 1111 will need additional leaves marked "invalid"
   for 10, 110 and 1110. Even if we don't allow allocate space for
   these leves, we need nodes for the prefixes, i.e., in all we need
   five nodes, not three, namely root, 0, 1, 11, 111, 1111. */
#define HUFFMAN_FUDGE_FACTOR 10
#define HUFFMAN_SIZE(n) (2*(n) - 1 + HUFFMAN_FUDGE_FACTOR)

int
huffman_table(struct huffman *table,
	      unsigned n, const unsigned char *lengths);


const struct huffman *
huffman_decode_bit(const struct huffman *table, unsigned bit, int *value);
